import 'dart:html';
import 'package:rad_textures/rad_textures.dart';
import 'package:vector_math/vector_math.dart';

void main() {
  window.onLoad.listen((e) { _create(); });
}

int map_double_to_int(double val) {
  assert(val >= -1.0);
  assert(val <= 1.0);
  int offset = 0;
  if(val < 0) {
    offset = (val * 127).toInt();
  } else {
    offset = (val * 128).toInt();
  }
  return (127 + offset);
}

void _plot_bumps(String canvas_id, TSampler noise_callback) {
  CanvasElement canvas = querySelector("#$canvas_id");
  CanvasRenderingContext2D ctx = canvas.getContext('2d');;
  ImageData imageData = ctx.getImageData(0, 0, canvas.width, canvas.height);
  double scale = 0.01;
  Vector3 vec_00 = new Vector3.zero();
  Vector3 vec_10 = new Vector3.zero();
  Vector3 vec_01 = new Vector3.zero();
  Vector3 normal = new Vector3.zero();
  for(int x = 0; x < canvas.width; x++) {
    for(int y = 0; y < canvas.height; y++) {
      double sample_00 = noise_callback(new Vector3(x * scale, y * scale, 0.0));
      double sample_10 = noise_callback(new Vector3((x + 1) * scale, y * scale, 0.0));
      double sample_01 = noise_callback(new Vector3(x * scale, (y + 1) * scale, 0.0));
      vec_00.setValues(x * scale, y * scale, sample_00);
      vec_10.setValues((x + 1) * scale, y * scale, sample_10);
      vec_01.setValues(x * scale, (y + 1) * scale, sample_01);
      vec_10.sub(vec_00);vec_10.normalize();
      vec_01.sub(vec_00);vec_01.normalize();
      vec_10.crossInto(vec_01, normal);
      normal.normalize();
      imageData.data[(y * canvas.width + x) * 4 + 0] = map_double_to_int(normal.x);
      imageData.data[(y * canvas.width + x) * 4 + 1] = map_double_to_int(normal.y);
      imageData.data[(y * canvas.width + x) * 4 + 2] = map_double_to_int(normal.z);
      imageData.data[(y * canvas.width + x) * 4 + 3] = 255;
    }
  }
  ctx.putImageData(imageData, 0, 0, 0, 0, canvas.width, canvas.height);
}

void _plot_noise(String canvas_id, TSampler noise_callback)
{
  CanvasElement canvas = querySelector("#$canvas_id");
  CanvasRenderingContext2D ctx = canvas.getContext('2d');;
  ImageData imageData = ctx.getImageData(0, 0, canvas.width, canvas.height);
  double scale = 0.01;
  for(int x = 0; x < canvas.width; x++) {
    for(int y = 0; y < canvas.height; y++) {
      double sample = noise_callback(new Vector3(x * scale, y * scale, 0.0));
      imageData.data[(y * canvas.width + x) * 4 + 0] = 0;
      imageData.data[(y * canvas.width + x) * 4 + 1] = 
          FASTFLOOR((sample * 0.5 + 0.5) * 255);
      imageData.data[(y * canvas.width + x) * 4 + 2] = 0;
      imageData.data[(y * canvas.width + x) * 4 + 3] = 255;
    }
  }
  ctx.putImageData(imageData, 0, 0, 0, 0, canvas.width, canvas.height);  
}

double pnoise3_map(Vector3 v) {
  const double scale = 3.0;
  return pnoise3(v.x * scale, v.y * scale, v.z * scale, 8, 8, 8); 
}
double noise3_map(Vector3 v) { return noise3(v.x, v.y, v.z); }
double RidgedMultifractal_Default_map(Vector3 v) { return RidgedMultifractal_Default(v.x, v.y, v.z); }
double turbulence_default_map(Vector3 v) { return turbulence_default(v.x, v.y, v.z); }
double FCellNoise3_map(Vector3 v) { return FCellNoise3(v.x, v.y, v.z); }
double fBm_default_map(Vector3 v) { return fBm_default(v.x, v.y, v.z); }
double HybridMultifractal_map(Vector3 v) { return HybridMultifractal(v, 0.63, 0.75, 5.0, 0.3); }

void _create() {
  _plot_noise("canvas_periodic_noise", pnoise3_map);
  _plot_bumps("canvas_periodic_noise_bumps", pnoise3_map);
  _plot_noise("canvas_simplex_noise", noise3_map);
  _plot_bumps("canvas_simplex_noise_bumps", noise3_map);
  _plot_noise("canvas_ridged_multifractal", RidgedMultifractal_Default_map);
  _plot_bumps("canvas_ridged_multifractal_bumps", RidgedMultifractal_Default_map);
  _plot_noise("canvas_turbulence", turbulence_default_map);
  _plot_bumps("canvas_turbulence_bumps", turbulence_default_map);
  _plot_noise("canvas_cell_noise", FCellNoise3_map);
  _plot_noise("canvas_fbm", fBm_default_map);
  _plot_bumps("canvas_fbm_bumps", fBm_default_map);
  _plot_noise("canvas_hybrid_multifractal", HybridMultifractal_map);
  _plot_bumps("canvas_hybrid_multifractal_bumps", HybridMultifractal_map);
}
